"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _constants = require("../constants");

var _importResource = require("../import-resource");

var _resolveResource = require("../resolve-resource");

var _declare_runtime_env = require("../declare_runtime_env");

var _getLog = require("../get-log");

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

// TODO: get rid of that entryPointMarker. There is more convenient ways to determine script location
const emitStaticImport = async runtime => {
  const imports = [`import '$resolve.guardOnlyServer'`, `import serverAssemblies from '$resolve.serverAssemblies'`, `import { getLog } from '@resolve-js/runtime-base'`];
  const constants = [``];
  const exports = [``];
  (0, _importResource.importResource)({
    resourceName: `runtimeEntry`,
    resourceValue: runtime,
    runtimeMode: _constants.RUNTIME_ENV_OPTIONS_ONLY,
    importMode: _constants.RESOURCE_ANY,
    instanceMode: _constants.IMPORT_INSTANCE,
    imports,
    constants
  });
  constants.push(`
    const log = getLog('backendStaticImport')
    const { entry, execMode } = runtimeEntry
    let worker
    entry(serverAssemblies).then((w) => {
      worker = w
      if (execMode === 'immediate') {
        log.debug('"execMode" set to "immediate", executing worker')
        worker().catch((error) => log.error(error))
        log.debug('worker now running')
      }
    })
  `);
  exports.push(`export { entryPointMarker } from '@resolve-js/runtime-base'`);
  exports.push(`export default worker`);
  return [...imports, ...constants, ...exports].join('\r\n');
};

const emitDynamicImport = async runtime => {
  const {
    result,
    imported,
    isPackage
  } = (0, _resolveResource.resolveResource)(runtime.module);
  const moduleImport = isPackage ? imported : 'default'; // eslint-disable-next-line no-undef

  const runtimeModule = await Promise.resolve(`${result}`).then(s => _interopRequireWildcard(require(s)));
  const {
    execMode
  } = await runtimeModule[moduleImport]();
  return `
    import '$resolve.guardOnlyServer'
    import { getLog, entryPointMarker } from '@resolve-js/runtime-base'
    
    const log = getLog('backendDynamicImportEntry')
    const runtimeOptions = ${(0, _declare_runtime_env.injectRuntimeEnv)(runtime.options)}

    const handler = async (...args) => {
      try {
        if(!global.initPromise) {
          const interopRequireDefault = require('@babel/runtime/helpers/interopRequireDefault')
          global.serverAssemblies = interopRequireDefault(
            require('$resolve.serverAssemblies')
          ).default
          
          const entryFactory = interopRequireDefault(
            require('${result}')
          ).${moduleImport} 
          
          const { entry } = entryFactory(runtimeOptions)
          
          global.entry = entry
          global.initPromise = entry(serverAssemblies)
        }
        const worker = await initPromise
        return await worker(...args)
      } catch(error) {
        log.error('Fatal error: ', error)
        throw error
      }
    }
    ${execMode === 'immediate' ? `
          log.debug('"execMode" set to "immediate", executing worker') 
          handler().catch((error) => log.error(error))
          ` : ''}
    export { entryPointMarker }
    export default handler
  `;
};

const importEntry = async ({
  resolveConfig,
  isClient
}) => {
  const log = (0, _getLog.getLog)(`$resolve.backendEntry`);

  if (isClient) {
    throw new Error(`${_constants.message.serverAliasInClientCodeError}$resolve.entry`);
  }

  const runtime = resolveConfig.runtime;
  const code = runtime.options && runtime.options.importMode === 'dynamic' ? await emitDynamicImport(runtime) : await emitStaticImport(runtime);
  log.verbose(`Runtime import generated code:`);
  log.verbose(code);
  return code;
};

var _default = importEntry;
exports.default = _default;
//# sourceMappingURL=$resolve.backendEntry.js.map