"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _constants = require("../constants");

var _declare_runtime_env = require("../declare_runtime_env");

var _importResource = require("../import-resource");

const importMonitoringAdapters = ({
  resolveConfig,
  isClient
}) => {
  if (isClient) {
    throw new Error(`${_constants.message.serverAliasInClientCodeError}.monitoringAdapters`);
  }

  const imports = [`import '$resolve.guardOnlyServer'`];
  const constants = [``];
  const exports = [`const monitoringAdapters = {}`];
  const adapterIds = Object.keys(resolveConfig.monitoringAdapters);

  for (let index = 0; index < adapterIds.length; index++) {
    const id = adapterIds[index];
    const adapterConfig = resolveConfig.monitoringAdapters[id];
    constants.push(`const name_${index} = ${JSON.stringify(adapterConfig)}`);
    (0, _importResource.importResource)({
      resourceName: `factory_s_${index}`,
      resourceValue: adapterConfig,
      runtimeMode: _constants.RUNTIME_ENV_ANYWHERE,
      importMode: _constants.RESOURCE_ANY,
      instanceMode: _constants.IMPORT_CONSTRUCTOR,
      imports,
      constants
    });

    if (adapterConfig.constructor === Object && adapterConfig.options != null) {
      constants.push(`const options_s_${index} = ${(0, _declare_runtime_env.injectRuntimeEnv)(adapterConfig.options)}`);
    } else {
      constants.push(`const options_s_${index} = null`);
    }

    constants.push(`const factory_${index} = (...args) => {
        return factory_s_${index}(options_s_${index}, ...args)
      }`);
    exports.push(`monitoringAdapters['${id}'] = factory_${index}`);
  }

  exports.push(`export default monitoringAdapters`);
  return [...imports, ...constants, ...exports].join('\r\n');
};

var _default = importMonitoringAdapters;
exports.default = _default;
//# sourceMappingURL=$resolve.monitoringAdapters.js.map